module Dhaka
  class Evaluator

    def initialize(syntax_tree)
      @syntax_tree = syntax_tree
      @node_stack = []
    end

    def result
      evaluate(@syntax_tree)
    end

    def child_nodes
      @node_stack[-1]
    end

    private
  
    def self.inherited(evaluator)
      class << evaluator
        attr_accessor :grammar, :actions
      end
    end
  
    def self.define_evaluation_rules
      default_action = Proc.new { child_nodes[0] }
      self.actions = Hash.new { |hash, key| default_action }
      yield
      check_definitions
    end
  
    def self.check_definitions
      non_trivial_productions_with_rules_undefined = self.grammar.productions.select {|production| production.expansion.size != 1}.collect {|production| production.name} - self.actions.keys
      raise EvaluatorDefinitionError.new(non_trivial_productions_with_rules_undefined) unless non_trivial_productions_with_rules_undefined.empty?
    end
  
    def evaluate node
      return node if (ParseTreeLeafNode === node)
      @node_stack << node.child_nodes.collect {|child_node| evaluate(child_node)}
      proc = self.class.actions[node.production.name]
      result = self.instance_eval(&proc)
      @node_stack.pop
      result
    end

    def self.for_rule_named(name, &blk)
      self.actions[name] = blk
    end
  
    def self.method_missing(method_name, &blk)
      if method_name.to_s =~ /^for_*/
        rule_name = method_name.to_s[4..-1]
        self.for_rule_named(rule_name, &blk)
      end 
    end
    
  end

  class EvaluatorDefinitionError < StandardError
    def initialize(non_trivial_productions_with_rules_undefined)
      @non_trivial_productions_with_rules_undefined = non_trivial_productions_with_rules_undefined
    end
  
    def to_s
        result = "The following non-trivial productions do not have any evaluation rules defined:\n" 
        result << (@non_trivial_productions_with_rules_undefined).join("\n")
    end
  end
end