module Dhaka
  class Action
    attr_reader :action_code
  end

  class ShiftAction < Action
    attr_reader :destination_state
    def initialize destination_state
      @destination_state = destination_state
      @action_code = Proc.new do
        state_stack << destination_state
        []
      end
    end
    def compile_to_ruby_source
      "shift_to #{@destination_state.id}"
    end
    def to_s
      "Shift to #{@destination_state}"
    end
  end

  class ReduceAction < Action
    attr_reader :production
    def initialize(production)
      @production = production
      @action_code = Proc.new do
        composite_node = ParseTreeCompositeNode.new(production)

        production.expansion.each { |symbol| 
            state_stack.pop
            composite_node.child_nodes.unshift(node_stack.pop)
           }

        node_stack << composite_node

        unless composite_node.head_node?
          [production.symbol.name, current_token.grammar_symbol.name]
        else
          []
        end
      end
    end
    def compile_to_ruby_source
      "reduce_with '#{@production.name}'"
    end
    def to_s
      "Reduce with #{production}"
    end
  end
end