#!/usr/bin/env ruby
module Dhaka
  class Channel
    attr_reader :start_item, :end_item
    def initialize(grammar, start_item, end_item)
      @grammar = grammar
      @start_item = start_item
      @end_item = end_item
    end
    def propagate cargo
      diff = cargo - @end_item.lookaheadset
      @end_item.lookaheadset.merge(diff)
      !diff.empty?
    end
    def to_s
      "Channel from #{@start_item} to #{@end_item}"
    end
    def eql? other
      @start_item.eql?(other.start_item) and @end_item.eql?(other.end_item)
    end
    def hash
      @start_item.hash ^ @end_item.hash
    end
  end

  class SpontaneousChannel < Channel
    def to_s
      "Spontaneous " + super.to_s
    end
    def pump
      follow_index = @start_item.next_item_index + 1
      cargo = Set.new
      while follow_symbol = @start_item.production.expansion[follow_index]
        cargo += @grammar.first(follow_symbol)
        return propagate(cargo) unless follow_symbol.nullable
        follow_index += 1
      end
      cargo += @start_item.lookaheadset
      propagate cargo
    end
  end

  class PassiveChannel < Channel
    def to_s
      "Passive " + super.to_s
    end
    def pump
      propagate @start_item.lookaheadset
    end
  end
end