module Dhaka
  class ParseTreeCompositeNode
    attr_reader :production, :child_nodes
    def initialize(production)
      @production = production
      @child_nodes = []
    end  
    def linearize
      child_nodes.collect {|child_node| child_node.linearize}.flatten + [production.name]
    end
    def to_s
      "CompositeNode: #{production.symbol} --> [#{child_nodes.join(", ")}]"
    end
    def dot_name
      "Node#{object_id}"
    end

    def to_dot
      result = []
      result << ["digraph x {", "node [fontsize=\"10\" shape=box size=\"5\"]"] if head_node?
      label = self.production
      result << "#{dot_name} [label=\"#{label}\"]"
      child_nodes.each do |child|
        result << "#{dot_name} -> #{child.dot_name}"
        result << "#{child.to_dot}"
      end
      result << ['}'] if head_node?
      result.join("\n")
    end
    
    def head_node?
      production.symbol.name == START_SYMBOL_NAME
    end
  end

  class ParseTreeLeafNode 
    attr_reader :token
    def initialize(token)
      @token = token
    end
    def linearize
      []
    end
    def to_s
      "LeafNode: #{token}"
    end
    def dot_name
      "Node#{object_id}"
    end
    def to_dot
      label = "#{token}#{' : '+token.value.to_s if token.value}"
      "#{dot_name} [label=\"#{label}\"]"
    end
    def head_node?
      false
    end
  end
end