module Dhaka
  class ParserRun

    def initialize(grammar, start_state, token_stream)
      @grammar = grammar
      @node_stack = []
      @state_stack = [start_state]
      @token_stream = token_stream
      @current_token_index = 0
    end

    def run
      for_each_token_with_end do
        error = execute_action current_token.grammar_symbol.name
        return error if error
        node_stack << ParseTreeLeafNode.new(current_token)
        @current_token_index += 1
      end
      ParseSuccessResult.new(node_stack[0])
    end

    private

    attr_reader :state_stack, :token_stream, :node_stack, :current_token

    def for_each_token_with_end
      token_stream.each do |@current_token|
        yield
      end
      @current_token = Token.new(@grammar.end_symbol, nil)
      yield
    end

    def execute_action symbol_name
      action = state_stack[-1].actions[symbol_name]
      return ParseErrorResult.new(@current_token_index) unless action
      self.instance_eval(&action.action_code).each do |symbol_name|
        execute_action symbol_name
      end
      nil
    end

  end
end