#!/usr/bin/env ruby
require 'set'
module Dhaka
  class ParserState

    attr_accessor :items, :actions, :id
  
    @@state_id = 0
  
    def self.next_state_id
      result = @@state_id
      @@state_id += 1
      result
    end
  
    def initialize(parser, items, id=nil)
      @parser = parser
      @items = items
      @actions = {}
      @id =  id ? id : ParserState.next_state_id
    end

    def transition_items
      result = Hash.new {|h, k| h[k] = ItemSet.new()}
      for item in @items.values
        (result[item.next_symbol] << item) if item.next_symbol
      end
      result
    end

    def dot_name
      self.to_s
    end

    def to_dot
      label = self.items.values.join('\n')
      "#{dot_name} [label=\"#{label}\"]"
    end
    def compile_to_ruby_source
      result = "  at_state(#{@id}) {\n"
      actions.each do |symbol_name, action|
        result << "    for_symbol('#{symbol_name}') { #{action.compile_to_ruby_source} }\n"
      end
      result << "  }"
      result
    end
  
    def for_symbol symbol_name, &blk
      actions[symbol_name] = @parser.instance_eval(&blk)
    end
  
    def to_s
      "State#{id}"
    end
  
  end

  class ItemSet < Set
    def hash
      self.collect{|item| item.hash}.inject{|result, hashcode| result ^ hashcode}
    end
    def eql? other
      self == other
    end
  end
end