require 'test/unit'
require 'compiled_arithmetic_parser'
require 'arithmetic_evaluator'
require 'arithmetic_test_methods'

class TestArithmeticEvaluator < Test::Unit::TestCase
  include ArithmeticTestMethods
  
  def setup
    @min_func = Proc.new {|args| args.inject {|min, elem| min = (elem < min ? elem : min)}}
    @max_func = Proc.new {|args| args.inject {|max, elem| max = (elem > max ? elem : max)}}
  end


  def test_results_simple_arithmetic_given_tokens_and_syntax_tree_1
    
    token_stream = [token('n', 2), token('-', nil), token('n', 4)]
    syntax_tree = get_syntax_tree_with_compiled_arithmetic_parser(token_stream)
    assert_equal -2, ArithmeticEvaluator.new(syntax_tree, @min_func, @max_func).result
    
  end

  def test_results_simple_arithmetic_given_tokens_and_syntax_tree_2

    token_stream = [token('n', 2), token('-', nil), token('(', nil), token('n', 3), token('/', nil), token('n', 4), token(')', nil)]
    syntax_tree = get_syntax_tree_with_compiled_arithmetic_parser(token_stream)
    assert_equal 1.25, ArithmeticEvaluator.new(syntax_tree, @min_func, @max_func).result

  end

  def test_results_simple_arithmetic_given_tokens_and_syntax_tree_3

    token_stream = [token('n', 2), token('+', nil), token('(', nil), token('n', 3), token('/', nil), token('(', nil), token('n', 7), token('-', nil), token('n', 5), token(')', nil) , token(')', nil)]
    syntax_tree = get_syntax_tree_with_compiled_arithmetic_parser(token_stream)
    assert_equal 3.5, ArithmeticEvaluator.new(syntax_tree, @min_func, @max_func).result

  end

  def test_results_simple_arithmetic_given_tokens_and_syntax_tree_4

    token_stream = [token('n', 2), token('+', nil), token('h', nil), token('(', nil), token('n', 3), token(',', nil), token('n', 4), token(')', nil)]
    syntax_tree = get_syntax_tree_with_compiled_arithmetic_parser(token_stream)
    assert_equal 6, ArithmeticEvaluator.new(syntax_tree, @min_func, @max_func).result

  end

  def test_results_simple_arithmetic_given_tokens_and_syntax_tree_5

    token_stream = [token('n', 2), token('+', nil), token('l', nil), token('(', nil), token('n', 3), token(',', nil), token('n', 4), token(')', nil)]
    syntax_tree = get_syntax_tree_with_compiled_arithmetic_parser(token_stream)
    assert_equal 5, ArithmeticEvaluator.new(syntax_tree, @min_func, @max_func).result

  end

end