require "test/unit"
require "arithmetic_tokenizer"

class TestArithmeticTokenizer < Test::Unit::TestCase
  def test_returns_empty_array_for_empty_input
    assert_equal([], ArithmeticTokenizer.tokenize([]))
  end
  def test_tokenizes_given_a_string_input
    assert_equal([token('n', 2), token('-', nil), token('n', 4)], ArithmeticTokenizer.tokenize('2 - 4')) 
  end
  def test_a_longer_input
    actual = ArithmeticTokenizer.tokenize('2+(3 / (7 - 5))')
    assert_equal([token('n', 2), token('+', nil), token('(', nil), token('n', 3), token('/', nil), token('(', nil), token('n', 7), token('-', nil), token('n', 5), token(')', nil) , token(')', nil)], actual)
  end

  def test_another_input_with_multi_digit_numbers
    actual = ArithmeticTokenizer.tokenize('2034 +(3433 / (7 - 5))')
    assert_equal([token('n', 2034), token('+', nil), token('(', nil), token('n', 3433), token('/', nil), token('(', nil), token('n', 7), token('-', nil), token('n', 5), token(')', nil) , token(')', nil)], actual)
  end

  def test_an_input_with_unrecognized_characters
    assert_raise(Dhaka::UnrecognizedInputCharacterException) {ArithmeticTokenizer.tokenize('2+(3 / (7 -& 5))')}
  end

  def test_another_input_with_illegal_characters
    assert_raise(Dhaka::UnrecognizedInputCharacterException) {ArithmeticTokenizer.tokenize('2034 +(34b3 / (7 - 5))')}
  end

  def token(symbol_name, value)
    Dhaka::Token.new(ArithmeticGrammar.symbol_for_name(symbol_name), value)
  end
end