require "test/unit"
require "simple_grammar"
require 'compiled_arithmetic_parser'
require 'arithmetic_test_methods'

class TestCompiledParser < Test::Unit::TestCase
  include ArithmeticTestMethods

  def test_compiled_parser_generates_syntax_tree_for_simple_grammar
    grammar = SimpleGrammar
    parser = Dhaka::Parser.new(grammar)
    eval(parser.compile_to_ruby_source_as('Foo'))
    syntax_tree = Foo.parse(build_tokens(['(','n','-','(','n','-','n',')',')','-','n','#'], Foo.grammar)).syntax_tree
    assert_equal \
      ["literal",
       "term",
       "literal",
       "term",
       "literal",
       "subtraction",
       "parenthetized_expression",
       "subtraction",
       "parenthetized_expression",
       "term",
       "literal",
       "subtraction",
       "expression",
       "start"], syntax_tree.linearize
  end

  def test_compiled_parser_generates_syntax_tree_for_arithmetic_grammar
    parser_input = ['(','n','-','(','n','/','n','-','n',')','/','n',')']
    assert_equal \
      ["getting_literals",
       "factor",
       "term",
       "getting_literals",
       "factor",
       "getting_literals",
       "division",
       "term",
       "getting_literals",
       "factor",
       "subtraction",
       "unpacking_parenthetized_expression",
       "factor",
       "getting_literals",
       "division",
       "subtraction",
       "unpacking_parenthetized_expression",
       "factor",
       "term",
       "expression"], get_syntax_tree_with_compiled_arithmetic_parser(build_tokens(parser_input, CompiledArithmeticParser.grammar)).linearize
  end
  
  def test_parse_result_has_error_if_empty_token_array
    assert CompiledArithmeticParser.parse([]).has_error?
  end

  def test_parser_returns_error_result_with_index_of_bad_token_if_parse_error
    parse_result = CompiledArithmeticParser.parse(build_tokens(['(', '-', ')'], CompiledArithmeticParser.grammar))
    assert parse_result.has_error?
    assert_equal 1, parse_result.bad_token_index
  end

  def build_tokens(token_symbol_names, grammar)
    token_symbol_names.collect {|symbol_name| Dhaka::Token.new(grammar.symbol_for_name(symbol_name), nil)}
  end

end